// iGate v1.00 : Programme M5Stack LoRa.
// Ecrit par Frank LOBA.
// Date création : 20/05/2023.
// Dernière mise à jour : 21/05/2023.


/**** INCLUSION DES FICHIERS EN-TETE ****/
#include <M5Stack.h>
#include <M5LoRa.h>
#include <WiFi.h>

#include <M5ModemConfig.h>

#include <HTTPClient.h>


/**** DEFINITION DES CONSTANTES ****/
#define LORA_SCK  18
#define LORA_NSS  5
#define LORA_MISO 19  
#define LORA_MOSI 23
#define LORA_RST  26
#define LORA_IRQ  36

#define LORA_REG_MODEM_PHY_CONFIG_3_ADRESS 0x26  // See SEMTECH SX1276/77/78/79 Datasheet Rev. 6 - January 2019, page 91.
#define LORA_REG_MODEM_PHY_CONFIG_3_VALUE  0xC

#define LORA_FREQUENCE         433775000L
#define LORA_SPREADING_FACTOR  12
#define LORA_SIGNAL_BANDWIDTH  125000L
#define LORA_CODING_RATE_4     5
#define LORA_SYNC_WORD         0x12

#define LORA_START_BYTE_1 '<'
#define LORA_START_BYTE_2 0xFF
#define LORA_START_BYTE_3 0x01

// Il faut renseigner votre indacatif d'abord, puis vos coordonnees géographiques à la place des x.
#define BEACON "xxxxx-xx>APLG01,WIDE1-1,WIDE2-1,TCPIP*:!xxxx.xxNLxxxxx.xxE&iGate LoRa\n"

// Il faut renseigner le SSID et le mot de passe de votre réseau WIFI.
#define WIFI_SSID "xxxxxxxx"
#define WIFI_PASS "xxxxxxxx"

// Il faut renseigner votre indicatif APRS-IS d'acord, puis votre code d'accès à la place des x.
#define APRSIS_LOGGIN "user xxxxx-xx pass xxxxx vers M5StackAPRS 1.0\n"
#define APRSIS_SERVER "http://srvr.aprs-is.net:8080"


/**** DEFINITION DES VARIABLES GLOBALES ****/
Modem   Modem;

String  Trame;

HTTPClient  AprsIs;


/**** DEFINITION DE LA FONCTION D'INITIALISATION ****/
void setup()
{
  // Initialisation du système.
  M5.begin();
  M5.Power.begin();

  // Connexion du WIFI.
  if(!WiFi.begin(WIFI_SSID, WIFI_PASS))
  {
    M5.Lcd.println("Echec connexion WIFI !");
    delay(5000);
  }

  // Initialisation du son.
  M5.Speaker.begin();

  // Affectation des broches.
  SPI.begin(LORA_SCK, LORA_MISO, LORA_MOSI, LORA_NSS);
  LoRa.setPins(LORA_NSS, LORA_RST, LORA_IRQ);

  // Défintion de la fréquence et démarrage du LoRa
  if(!LoRa.begin(LORA_FREQUENCE))
  {
    M5.Lcd.println("Echec demarrage LoRa !");
    while(1);
  }

  // Définition des paramètres système pour LoRa.
  LoRa.setSpreadingFactor(LORA_SPREADING_FACTOR);
  LoRa.setSignalBandwidth(LORA_SIGNAL_BANDWIDTH);
  LoRa.setCodingRate4(LORA_CODING_RATE_4);
  LoRa.enableCrc();
  LoRa.setSyncWord(LORA_SYNC_WORD);

  Modem.writeRegister(LORA_REG_MODEM_PHY_CONFIG_3_ADRESS, LORA_REG_MODEM_PHY_CONFIG_3_VALUE);
}


/**** DEFINITION DE LA FONCTION D'EXECUTION ****/
void loop()
{
  M5.update();

  // Appuis sur le bouton A : Affichage de l'adresse IP.
  if(M5.BtnA.wasPressed())
  {
    // Effacement et initialisation de l'écran du M5Stack.
    M5.Lcd.clear();
    M5.Lcd.setCursor(0, 0);
    M5.Lcd.setTextSize(3);

    // Affichage des informations WIFI.
    M5.Lcd.println("Adresse IP");
    M5.Lcd.println(WiFi.localIP());
  }
    
  // Appuis sur le bouton B : Envoie de la balise APRS par Internet.
  if(M5.BtnB.wasPressed())
  {
    // Effacement et initialisation de l'écran du M5Stack.
    M5.Lcd.clear();
    M5.Lcd.setCursor(0, 0);
    M5.Lcd.setTextSize(2);

    // Affichage de "IS" à l'écran du M5Stack
    M5.Lcd.println("**** IS ****");

    // Constitution de la trame TNC2.
    Trame = BEACON;
    
    // Affichage de la trame TNC2 à l'écran du M5Stack
    M5.Lcd.println(Trame);

    // Ajout du loggin APRS-IS à la trame TNC2.
    Trame = APRSIS_LOGGIN + Trame;
    
    // Envoie de la trame TNC2 à APRS-IS si le WIFI est connecté.
    if(WiFi.status() == WL_CONNECTED)
    {
      // Connexion à APRS-IS.
      AprsIs.begin(APRSIS_SERVER);

      // Ajout de l'en-tête HTTP.
      AprsIs.addHeader("Accept-Type", "text/plain");
      AprsIs.addHeader("Content-Type", "application/octet-stream");

      // Envoie de la trame TNC2.
      int AprsIsCode = AprsIs.POST(Trame);

      // Affichage du code HTTP renvoyé par APRS-IS.
      M5.Lcd.print("Code APRS-IS : ");
      M5.Lcd.println(AprsIsCode);

      // Déconnexion de APRS-IS.
      AprsIs.end();
    }
  }

  // Appuis sur le bouton C : Envoie de la balise APRS par radio.
  if(M5.BtnC.wasPressed())
  {
    // Effacement et initialisation de l'écran du M5Stack.
    M5.Lcd.clear();
    M5.Lcd.setCursor(0, 0);
    M5.Lcd.setTextSize(2);

    // Affichage de "TX" à l'écran du M5Stack
    M5.Lcd.println("**** TX ****");

    // Constitution de la trame TNC2.
    Trame = BEACON;

    // Affichage de la trame TNC2 à l'écran du M5Stack
    M5.Lcd.println(Trame);
    
    // Commencement de l'émission LoRa.
    LoRa.beginPacket();
            
    // Emission de l'en-tête LoRa.
    LoRa.write(LORA_START_BYTE_1);
    LoRa.write(LORA_START_BYTE_2);
    LoRa.write(LORA_START_BYTE_3);
            
    // Emission de la trame TNC2 en LoRa.
    LoRa.write((const uint8_t *)Trame.c_str(), Trame.length());
            
    // Fin de l'émission LoRa.
    LoRa.endPacket();
  }

  
  // Réception des messages radio.  

  // Détection de la réception d'un paquet.
  int PacketSize = LoRa.parsePacket();
  if (PacketSize)
  {
    // Effacement et initialisation de l'écran du M5Stack.
    M5.Lcd.clear();
    M5.Lcd.setCursor(0, 0);
    M5.Lcd.setTextSize(2);

    // Affichage de "RX" à l'écran du M5Stack
    M5.Lcd.println("**** RX ****");

    // Initialisation de la trame TNC2.
    Trame = "";

    // Détection de l'en-tête LoRa.
    if(LoRa.available() && LoRa.read() == LORA_START_BYTE_1)
    if(LoRa.available() && LoRa.read() == LORA_START_BYTE_2)
    if(LoRa.available() && LoRa.read() == LORA_START_BYTE_3)
    
    // Lecture du paquet au format TNC2.
    while(LoRa.available())
    Trame += (char)LoRa.read();

    // Affichage de la trame au format TNC2 à l'écran.
    M5.Lcd.println(Trame);

    // Ajout du loggin APRS-IS à la trame TNC2.
    Trame = APRSIS_LOGGIN + Trame;
    
    // Envoie de la trame TNC2 à APRS-IS si le WIFI est connecté.
    if(WiFi.status() == WL_CONNECTED)
    {
      // Connexion à APRS-IS.
      AprsIs.begin(APRSIS_SERVER);

      // Ajout de l'en-tête HTTP.
      AprsIs.addHeader("Accept-Type", "text/plain");
      AprsIs.addHeader("Content-Type", "application/octet-stream");

      // Envoie de la trame TNC2.
      int AprsIsCode = AprsIs.POST(Trame);

      // Affichage du code HTTP renvoyé par APRS-IS.
      M5.Lcd.print("Code APRS-IS : ");
      M5.Lcd.println(AprsIsCode);

      // Déconnexion de APRS-IS.
      AprsIs.end();
    }
  }

  delay(20);
}
